<?php
/**
 * @package		Joomla.Plugin
 * @subpackage	System.Jblibrary
 * @author		Joomla Bamboo - design@joomlabamboo.com
 * @copyright 	Copyright (c) 2012 Joomla Bamboo. All rights reserved.
 * @license		GNU General Public License version 2 or later
 * @version		2.0.1
 */

// no direct access
defined('_JEXEC') or die('Restricted access');
defined('DS') or define('DS', DIRECTORY_SEPARATOR);

defined('JBLIBRARY_MEDIA_PATH') or define('JBLIBRARY_MEDIA_PATH', JPATH_SITE.DS.'media'.DS.'plg_jblibrary');
define('JBLIBRARY_HELPER_PATH', JBLIBRARY_MEDIA_PATH.DS.'helpers');
define('NEXT_SCRIPT_TOKEN', '#$NEXT_SCRIPT$#');

/** Thanks to onejQuery for being the inspiration of our unique jQuery function **/
/** ensure this file is being included by a parent file */
jimport( 'joomla.plugin.plugin' );

if (version_compare(JVERSION, '3.0'))
{
	jimport( 'joomla.filesystem.folder' );
}

// Load helpers
require_once(JBLIBRARY_HELPER_PATH.DS.'library.php');

class plgSystemJblibrary extends JPlugin
{
	public function __construct(&$subject, $config)
	{
		if (JFactory::getApplication()->isAdmin()) return;

		parent::__construct($subject, $config);

		$this->loadLanguage();

		$this->browserIsIE60 = LibraryHelper::browserIsIE60();
	}

	public function onBeforeRender()
	{
		$addScrolltop   = (bool)$this->params->get('scrollTop', 1);
		$scrollTopStyle = $this->params->get('scrollStyle', 'dark');

		if ($addScrolltop && !$this->browserIsIE60)
		{
			$doc  =& JFactory::getDocument();

			if ($scrollTopStyle === 'dark')
			{
				$doc->addStyleDeclaration('#toTop {width:100px;z-index: 10;border: 1px solid #333; background:#121212; text-align:center; padding:5px; position:fixed; bottom:0px; right:0px; cursor:pointer; display:none; color:#fff;text-transform: lowercase; font-size: 0.7em;}');
			}
			elseif ($scrollTopStyle === 'light')
			{
				$doc->addStyleDeclaration('#toTop {width:100px;z-index: 10;border: 1px solid #eee; background:#f7f7f7; text-align:center; padding:5px; position:fixed; bottom:0px; right:0px; cursor:pointer; display:none;  color:#333;text-transform: lowercase; font-size: 0.8em;}');
			}
		}
	}

	public function onAfterRender()
	{
		if (JFactory::getApplication()->isAdmin()) return;

		// Params
		$this->browserIsIE60     = LibraryHelper::browserIsIE60();
		$loadJQuery              = $this->params->get('loadJQuery', 1) == 1;
		$jQuerySource            = $this->params->get('source', 'google');
		$jQueryVersion           = $this->params->get('jQueryVersion', JBLIBRARY_DEFAULT_JQUERY);
		$jQueryNoConflict        = (bool)$this->params->get('noConflict', 1);
		$removeOtherJQuery       = (bool)$this->params->get('jqunique', 0);
		$otherJQueryRegex        = $this->params->get('jqregex', '([\/a-zA-Z0-9_:\.-]*)jquery([0-9\.-]|min|pack)*?.js');
		$handleMootools          = (bool)$this->params->get('handleMootools', 0);
		$addCustomScripts        = trim($this->params->get('addCustomScripts', ''));
		$stripMootools           = (bool)$this->params->get('stripMootools', 0);
		$stripMootoolsMore       = (bool)$this->params->get('stripMootoolsMore', 0);
		$replaceMootools         = (bool)$this->params->get('replaceMootools', 0);
		$replaceMootoolsMore     = (bool)$this->params->get('replaceMootoolsMore', 0);
		$ie6Warning              = (bool)$this->params->get('ie6Warning', 1);
		$mootoolsPathReplace     = $this->params->get('mootoolsPath', 'http://ajax.googleapis.com/ajax/libs/mootools/' . JBLIBRARY_DEFAULT_MOOTOOLS . '/mootools-yui-compressed.js');
		$mootoolsMorePathReplace = $this->params->get('mootoolsMorePath', '/media/system/js/mootools-more.js');
		$addScrolltop            = (bool)$this->params->get('scrollTop', 1);
		$stripCustomScripts      = (bool)$this->params->get('stripCustom', 0);
		$customScriptsToStrip    = trim($this->params->get('customScripts', ''));
		$scrollTopTextTranslate  = (bool)$this->params->get('scrollTextTranslate', 1);
		$scrollTopText           = $this->params->get('scrollText', '^ Back To Top');
		$scrollTopText           = $scrollTopTextTranslate ? JText::_($scrollTopText) : $scrollTopText;
		$lazyLoadImages          = (bool)$this->params->get('lazyLoad', 1);
		$lazyLoadSelector        = $this->params->get('llSelector', 'img');

		$jQueryFromCDN           = $jQuerySource !== 'local';
		$mootoolsFromCDN         = $replaceMootools && LibraryHelper::isExternalPath($mootoolsPathReplace);
		$mootoolsMoreFromCDN     = $replaceMootoolsMore && LibraryHelper::isExternalPath($mootoolsMorePathReplace);
		$loadScriptFromCDN       = $jQueryFromCDN || $mootoolsFromCDN || $mootoolsFromCDN;

		// Backward param compatibility
		if ($jQueryVersion === 'none')
		{
			$loadJQuery = false;
			$jQueryVersion = 'latest';
		}

		// There is no "latest" local version
		if ($jQueryVersion === 'latest' && $jQuerySource === 'local')
		{
			$jQueryVersion = JBLIBRARY_DEFAULT_JQUERY;
		}

		$jQueryPath = LibraryHelper::getJQueryPath($jQueryVersion, $jQuerySource);

		if (empty($lazyLoadSelector))
		{
			$lazyLoadSelector = 'img';
		}

		// Add custom scripts?
		if (!empty($addCustomScripts))
		{
			// Sanitize value
			$addCustomScripts = preg_replace('/\n\s\'"/m', '', $addCustomScripts);

			$addCustomScripts = explode(',', $addCustomScripts);
		}

		// Remove custom scripts?
		if ($stripCustomScripts)
		{
			$customScriptsToStrip = preg_split("/[\s,]+/", $customScriptsToStrip);
		}

		$body =& JResponse::getBody();

		// Find all external scripts in head and body
		$bodyScripts = array();
		$cleanScripts = array();
		$scriptDeclarations = array();
		preg_match_all('#<script[^>]*src[\s]*=[\"\']([^\"\']*)[\"\'][^>]*>[^<]*<\/script>#', $body, $bodyScripts);

		// Check if need to add custom scripts and inject it into found body scripts
		if (!empty($addCustomScripts))
		{
			if (!is_array($addCustomScripts))
			{
				$addCustomScripts = array($addCustomScripts);
			}

			$bodyScripts[1] = array_merge($bodyScripts[1], $addCustomScripts);
		}

		if (!empty($bodyScripts))
		{
			// Remove explicit duplicated scripts
			$scriptPaths = array_unique($bodyScripts[1]);
			unset($bodyScripts[1]);

			// Temp vars to sort scripts later
			$pathMootools     = '';
			$pathMootoolsMore = '';
			$pathJoomlaCore   = '';

			// Check if there is Mootools, Mootools More, Core and remove Custom Scripts
			$i = 0;
			foreach ($scriptPaths as $script)
			{
				if (substr_count($script, 'mootools-core'))
				{
					// Should handle Mootools?
					if ($handleMootools)
					{
						if (!$stripMootools)
						{
							$firstMootols = empty($pathMootools);
							if ($replaceMootools)
							{
								$script = $mootoolsPathReplace;
							}

							// Avoid duplicated mootools-core
							if ($firstMootols)
							{
								$pathMootools = $script;
							}
						}
					}
					else
					{
						$pathMootools = $script;
					}

					unset($scriptPaths[$i]);
				}
				elseif (substr_count($script, 'mootools-more'))
				{
					if ($handleMootools)
					{
						if (!$stripMootoolsMore)
						{
							$firstMootolsMore = empty($pathMootoolsMore);
							if ($replaceMootoolsMore)
							{
								$script = $mootoolsMorePathReplace;
							}

							// Avoid duplicated mootools-more
							if ($firstMootolsMore)
							{
								$pathMootoolsMore = $script;
							}
						}
					}
					else
					{
						$pathMootoolsMore = $script;
					}

					unset($scriptPaths[$i]);
				}
				elseif (substr_count($script, '/core.js'))
				{
					$pathJoomlaCore = $script;
					unset($scriptPaths[$i]);
				}
				elseif ($stripCustomScripts && !empty($customScriptsToStrip))
				{
					// Remove specified scripts
					foreach($customScriptsToStrip as $deadScript)
					{
						$scriptRegex = "([\/a-zA-Z0-9_:\.-]*)" . trim($deadScript);
						if (preg_match("~{$scriptRegex}~", $script))
						{
							unset($scriptPaths[$i]);
						}
					}
				}
				elseif (substr_count($script, 'jquery-noconflict') && $jQueryNoConflict)
				{
					// Remove duplicated noConflict script
					unset($scriptPaths[$i]);
				}
				elseif ($removeOtherJQuery) // Remove other jQuery?
				{
					if (preg_match('~' . $otherJQueryRegex . '~', $script))
					{
						unset($scriptPaths[$i]);
					}
				}

				$i++;
			}

			/**
			 * Start script reorder:
			 *
			 * 1. Script Loader
			 * 2. Mootools
			 * 3. Mootools More
			 * 4. Joomla Core JS
			 * 5. jQuery
			 * 6. jQuery noConflict
			 * 7. Other JS scripts
			 * 8. Lazy Load Images
			 * 8. BadBrowser
			 * 10. Scroll Top Button
			 * 11. User Script Files
			 */

			// 1. Script Loader
			// Just if jQuery, Mootools or Mootools More are loaded from CDN or external path
			if ($loadScriptFromCDN)
			{
				$cleanScripts[] = JURI::base( true ) .'/media/plg_jblibrary/js/scriptloader.js';
			}

			// 2. Mootools
			if (!empty($pathMootools))
			{
				$cleanScripts[] = $pathMootools;
			}

			// 3. Mootools More
			// Add Mootools more just if we have Mootools
			if (!empty($pathMootoolsMore))
			{
				$cleanScripts[] = $pathMootoolsMore;
			}

			// 4. Joomla Core JS
			if (!empty($pathJoomlaCore))
			{
				$cleanScripts[] = $pathJoomlaCore;
			}

			// Should load jQuery?
			if ($loadJQuery)
			{
				// 5. jQuery
				$cleanScripts[] = $jQueryPath;

				// 6. jQuery noConflict
				if ($jQueryNoConflict)
				{
					$cleanScripts[] = JBLIBRARY_JQUERY_PATH . 'jquery-noconflict.js';
				}
			}

			// 7. Other JS Scripts
			if (!empty($scriptPaths))
			{
				$cleanScripts = array_merge($cleanScripts, $scriptPaths);
				unset($scriptPaths);
			}

			// 8. Lazy Load Images
			if ($lazyLoadImages)
			{
				$cleanScripts[] = JBLIBRARY_JQUERY_PATH . 'jquery-lazyload.js';

				$scriptDeclarations[] = '
					jQuery(document).ready(function() {
						jQuery("' . $lazyLoadSelector . '").lazyload({
							effect : "fadeIn"
						});
					});
				';
			}

			// 9. BadBrowser
			if ($this->browserIsIE60)
			{
				if ($ie6Warning)
				{
					$cleanScripts[] = JBLIBRARY_JQUERY_PATH . 'jquery-badbrowser.js';
				}
			}
			else
			{
				// 10. Scroll To Top if Not IE6
				if ($addScrolltop)
				{
					$scriptDeclarations[] .= '
						jQuery(document).ready(function() {
							jQuery(function () {
								var scrollDiv = document.createElement("div");
								jQuery(scrollDiv).attr("id", "toTop").html("' . $scrollTopText . '").appendTo("body");
								jQuery(window).scroll(function () {
									if (jQuery(this).scrollTop() != 0) {
										jQuery("#toTop").fadeIn();
									} else {
										jQuery("#toTop").fadeOut();
									}
								});
								jQuery("#toTop").click(function () {
									jQuery("body,html").animate({
										scrollTop: 0
									},
									800);
								});
							});
						});
					';
				}
			}

			// 11. User Script Files
			$userFiles = JFolder::files('media'.DS.'plg_jblibrary'.DS.'user', 'js', false, true);
			if ($userFiles)
			{
				if (!empty($userFiles))
				{
					foreach ($userFiles as $file)
					{
						$cleanScripts[] = (string) JURI::root(true) . '/' . $file;
					}
				}
				unset($userFiles, $file);
			}

			// Remove old external script declarations
			$body = str_replace($bodyScripts[0], '', $body);

			// Get explicit script tags
			preg_match_all("/<script[^>]*>(.*)<\/script>/Uis", $body, $internalJS);
			if (!empty($internalJS[0]))
			{
				// Remove old script  declaration
				$body = str_replace($internalJS[0], '', $body);

				$scriptDeclarations = array_merge($internalJS[1], $scriptDeclarations);
			}

			$tags = '';

			/**
			 * Process cleaned script list.
			 *
			 * If use CDN, every script will be added inside a callback.
			 * That callback is called when the loader succesfull load the script.
			 * If can not load it, so the fallback URL will be used.
			 * After load the fallback script, the callback is called.
			 *
			 * The NEXT_SCRIPT_TOKEN is used (replaced) to insert each script loader inside the prior callback param.
			 */
			$loadScriptStack = NEXT_SCRIPT_TOKEN;
			if (!empty($cleanScripts))
			{
				$tags = '';

				// Prepare fallback scripts if there is any script from a CDN
				$fallbackScripts = array();
				if ($loadScriptFromCDN)
				{
					// Check if the script has a local copy inside fallback folder
					$fallbackScripts = JFolder::files('media'.DS.'plg_jblibrary'.DS.'js'.DS.'fallback', 'js', false, true);
				}

				$firstScript = true;
				foreach ($cleanScripts as $scriptURL)
				{
					// Check if the JS needs to be loaded using Script Loader - CDN Fallback
					// First script is the loader, so ignore it
					if ($loadScriptFromCDN && !$firstScript)
					{
						// Load using the script loader

						// If is Mootools from CDN, use a fallback script
						$fallbackURL = '';
						if (($scriptURL === $mootoolsPathReplace) && $mootoolsFromCDN)
						{
							$fallbackURL = '/media/system/js/mootools-core.js';
						}
						// If is Mootools More from CDN, use a fallback script
						elseif (($scriptURL === $mootoolsMorePathReplace) && $mootoolsMoreFromCDN)
						{
							$fallbackURL = '/media/system/js/mootools-more.js';
						}
						// If is jQuery from CDN, use a fallback script
						elseif (($scriptURL === $jQueryPath) && $jQueryFromCDN)
						{
							$fallbackURL = LibraryHelper::getJQueryPath(JBLIBRARY_DEFAULT_JQUERY, 'local');
						}
						else
						{
							// Get a fallback for any external script with a local copy
							if (!empty($fallbackScripts))
							{
								$i = 0;
								foreach ($fallbackScripts as $fallbackScript)
								{
									if (LibraryHelper::isExternalPath($scriptURL)
											&& basename($scriptURL) === basename($fallbackScript))
									{
										$fallbackURL = $fallbackScript;
										unset($fallbackScripts[$i]);
									}
									$i++;
								}
							}
						}

						$tmpJS = "loadScript('{$scriptURL}','{$fallbackURL}',function(){" . NEXT_SCRIPT_TOKEN . "});";
						$loadScriptStack = str_replace(NEXT_SCRIPT_TOKEN, $tmpJS, $loadScriptStack);
					}
					else
					{
						// Load as normal script tag
						$tags .= "<script src=\"{$scriptURL}\" type=\"text/javascript\"></script>\n";
						$firstScript = false;
					}
				}

				// Inject scripts into head, or just the script loader
				$body = str_replace('</head>', $tags . '</head>', $body);
			}

			// Add the explicit script declarations to the end of body
			if (!empty($scriptDeclarations))
			{
				$explicitScripts = '';
				foreach ($scriptDeclarations as $script)
				{
					// Add all explicit scripts together, to the same script tag
					if ($loadScriptStack !== NEXT_SCRIPT_TOKEN)
					{
						$explicitScripts .= "{$script}\n";
					}
					else
					{
						$explicitScripts .= "<script type=\"text/javascript\">{$script}</script>\n";
					}
				}

				// If is not using script loader, so add all explicit scripts to the end of body
				if (!($loadScriptStack !== NEXT_SCRIPT_TOKEN))
				{
					$body = str_replace('</body>', $explicitScripts . '</body>', $body);
					$explicitScripts = '';
				}
			}

			// Inject the script loader stack as the last script to body
			if ($loadScriptStack !== NEXT_SCRIPT_TOKEN) // Empty?
			{
				$lastCallback = '';
				if (!empty($explicitScripts))
				{
					// All that explicit scripts to the last callback, so they will be called after all scripts are loaded
					$lastCallback = $explicitScripts;
				}


				// Fix last loader callback as empty param
				$loadScriptStack = str_replace(NEXT_SCRIPT_TOKEN, $lastCallback, $loadScriptStack);
				// Add container script tag
				$loadScriptStack = "<script type=\"text/javascript\">{$loadScriptStack}</script>";
				// Inject into body
				$body = str_replace('</body>', $loadScriptStack . '</body>', $body);
			}

			unset($tags, $scriptDeclarations, $script, $cleanScripts);
		}

		JResponse::setBody($body);

		return true;
	}
}
?>
